@@ -180,9 +180,11 @@ public:
    void  Sort(sort_e newsort, bool casefold=false, bool reverse=false);
    void  Unsort();
    void	 SortByPatternList(const char *list_c);
+   void	 ReverseSort();
 
    void	 Exclude(const char *prefix,const PatternSet *x);
    void	 ExcludeDots();
+   void	 ExcludeCompound();
    void	 ExcludeUnaccessible();
 
    void	 rewind() { ind=0; }
