@@ -39,6 +39,8 @@ class MirrorJob : public Job
       TARGET_REMOVE_OLD,
       TARGET_REMOVE_OLD_FIRST,
       TARGET_CHMOD,
+      TARGET_MKDIR,
+      SOURCE_REMOVING_SAME,
       FINISHING,
       LAST_EXEC,
       DONE
@@ -54,7 +56,10 @@ class MirrorJob : public Job
    long long bytes_to_transfer;
    Ref<FileSet> target_set;
    Ref<FileSet> source_set;
+   Ref<FileSet> target_set_recursive;
+   Ref<FileSet> source_set_recursive;
    Ref<FileSet> to_transfer;
+   Ref<FileSet> to_mkdir;
    Ref<FileSet> same;
    Ref<FileSet> to_rm;
    Ref<FileSet> to_rm_mismatched;
@@ -155,7 +160,15 @@ class MirrorJob : public Job
    void TransferFinished(Job *j);
    void JobFinished(Job *j);
 
-   off_t GetBytesCount() { return bytes_transferred+Job::GetBytesCount(); }
+   off_t GetBytesCount() {
+      long long bytes_count=Job::GetBytesCount();
+      if(!parent_mirror) {
+	 // bytes_transferred is cumulative over the mirror tree,
+	 // add it on the top only
+	 bytes_count+=bytes_transferred;
+      }
+      return bytes_count;
+   }
    double GetTimeSpent() { return transfer_time_elapsed+(transfer_count>0?now-root_mirror->transfer_start_ts:0.); }
 
 public:
@@ -180,6 +193,7 @@ public:
       NO_EMPTY_DIRS=1<<16,
       DEPTH_FIRST=1<<17,
       ASCII=1<<18,
+      SCAN_ALL_FIRST=1<<19,
    };
 
    void SetFlags(int f,bool v)
