@@ -54,6 +54,7 @@
 #include "CopyJob.h"
 #include "OutputJob.h"
 #include "echoJob.h"
+#include "EditJob.h"
 
 #include "misc.h"
 #include "alias.h"
@@ -78,21 +79,16 @@
 
 History	 cwd_history;
 
-CMD(alias); CMD(anon);   CMD(cd);      CMD(debug);	CMD(du);
-CMD(exit);  CMD(get);    CMD(help);    CMD(jobs);
-CMD(kill);  CMD(lcd);    CMD(ls);      CMD(cls);
-CMD(open);  CMD(pwd);    CMD(set);     CMD(eval);
-CMD(shell); CMD(source); CMD(user);    CMD(rm);
-CMD(wait);  CMD(subsh);  CMD(mirror);  CMD(local);
-CMD(mv);    CMD(cat);    CMD(cache);   CMD(ln);
-CMD(mkdir); CMD(scache); CMD(mrm);
-CMD(ver);   CMD(close);  CMD(bookmark);CMD(lftp);
-CMD(echo);  CMD(suspend);CMD(sleep);   CMD(slot);
-CMD(at);    CMD(find);   CMD(command); CMD(module);
-CMD(lpwd);  CMD(glob);	 CMD(chmod);   CMD(queue);
-CMD(repeat);CMD(get1);   CMD(tasks);   CMD(torrent);
-
-CMD(empty); CMD(notempty); CMD(true); CMD(false);
+CMD(alias); CMD(anon); CMD(at); CMD(bookmark); CMD(cache); CMD(cat);
+CMD(cd); CMD(chmod); CMD(close); CMD(cls); CMD(command); CMD(debug);
+CMD(du); CMD(echo); CMD(edit); CMD(eval); CMD(exit); CMD(find); CMD(get);
+CMD(get1); CMD(glob); CMD(help); CMD(jobs); CMD(kill); CMD(lcd); CMD(lftp);
+CMD(ln); CMD(local); CMD(lpwd); CMD(ls); CMD(mirror); CMD(mkdir);
+CMD(module); CMD(mrm); CMD(mv); CMD(open); CMD(pwd); CMD(queue);
+CMD(repeat); CMD(rm); CMD(scache); CMD(set); CMD(shell); CMD(sleep);
+CMD(slot); CMD(source); CMD(subsh); CMD(suspend); CMD(tasks); CMD(torrent);
+CMD(user); CMD(ver); CMD(wait); CMD(empty); CMD(notempty); CMD(true);
+CMD(false);
 
 #ifdef MODULE_CMD_MIRROR
 # define cmd_mirror 0
@@ -238,6 +234,11 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -s, --summarize       display only a total for each argument\n"
 	 "     --exclude=PAT     exclude files that match PAT\n")},
    {"echo",    cmd_echo,   0},
+   {"edit",    cmd_edit,   N_("edit [OPTS] <file>"),
+	 N_("Retrieve remote file to a temporary location, run a local editor on it\n"
+	 "and upload the file back if changed.\n"
+	 " -k  keep the temporary file\n"
+	 " -o <temp>  explicit temporary file location\n")},
    {"eval",    cmd_eval,   0},
    {"exit",    cmd_exit,   N_("exit [<code>|bg]"),
 	 N_("exit - exit from lftp or move to background if jobs are active\n\n"
@@ -1815,6 +1816,58 @@ CMD(get)
    return j;
 }
 
+CMD(edit)
+{
+   /* Download specified remote file into a temporary local file; run an
+    * editor on it and upload the file back if changed. Remove the temp file. */
+
+   const char *op=args->a0();
+   xstring temp_file;
+   bool keep_temp=false;
+
+   int opt;
+   while((opt=args->getopt("ok"))!=EOF)
+   {
+      switch(opt)
+      {
+      case('o'):
+	 temp_file.set(optarg);
+	 break;
+      case('k'):
+	 keep_temp=true;
+	 break;
+      case('?'):
+      err:
+	 eprintf(_("Try `help %s' for more information.\n"),op);
+	 return 0;
+      }
+   }
+   args->rewind();
+   if(args->count()<=1) {
+      eprintf(_("File name missed. "));
+      goto err;
+   }
+   const char *file=args->getarg(1);
+   if(!temp_file) {
+      ParsedURL u(file);
+      temp_file.set(basename_ptr(u.proto?u.path.get():file));
+      // make temp file name by substituting node name and PID after the first dot.
+      xstring temp_str;
+      temp_str.setf("%s-%u.",get_nodename(),(unsigned)getpid());
+      int point=temp_file.instr('.');
+      temp_file.set_substr(point>=0?point+1:0,0,temp_str);
+      temp_file.set_substr(0,0,"/");
+      xstring temp_dir(dir_file(get_lftp_cache_dir(),"edit"));
+      mkdir(temp_dir,0700);
+      temp_file.set_substr(0,0,temp_dir);
+      if(access(temp_file,F_OK)!=-1)
+	 keep_temp=true;
+   }
+   EditJob *j=new EditJob(session->Clone(),file,temp_file);
+   j->KeepTempFile(keep_temp);
+   return j;
+}
+
 CMD(shell)
 {
    Job *j;
@@ -2046,13 +2099,12 @@ CMD(debug)
 	 debug_file_name=optarg;
 	 if(fd!=-1)
 	    close(fd);
-	 fd=open(debug_file_name,O_WRONLY|O_CREAT|O_APPEND,0600);
+	 fd=open(debug_file_name,O_WRONLY|O_CREAT|O_APPEND|O_NONBLOCK,0600);
 	 if(fd==-1)
 	 {
 	    perror(debug_file_name);
 	    return 0;
 	 }
-	 fcntl(fd,F_SETFL,O_NONBLOCK);
 	 fcntl(fd,F_SETFD,FD_CLOEXEC);
 	 break;
       case 'p':
@@ -2534,7 +2586,7 @@ CMD(scache)
    return 0;
 }
 
-void CmdExec::print_cmd_help(const char *cmd)
+bool CmdExec::print_cmd_help(const char *cmd)
 {
    const cmd_rec *c;
    int part=find_cmd(cmd,&c);
@@ -2544,30 +2596,31 @@ void CmdExec::print_cmd_help(const char *cmd)
       if(c->long_desc==0 && c->short_desc==0)
       {
 	 printf(_("Sorry, no help for %s\n"),cmd);
-	 return;
+	 return true;
       }
       if(c->short_desc==0 && !strchr(c->long_desc,' '))
       {
 	 printf(_("%s is a built-in alias for %s\n"),cmd,c->long_desc);
 	 print_cmd_help(c->long_desc);
-	 return;
+	 return true;
       }
       if(c->short_desc)
 	 printf(_("Usage: %s\n"),_(c->short_desc));
       if(c->long_desc)
 	 printf("%s",_(c->long_desc));
-      return;
+      return true;
    }
    const char *a=Alias::Find(cmd);
    if(a)
    {
       printf(_("%s is an alias to `%s'\n"),cmd,a);
-      return;
+      return true;
    }
    if(part==0)
       printf(_("No such command `%s'. Use `help' to see available commands.\n"),cmd);
    else
       printf(_("Ambiguous command `%s'. Use `help' to see available commands.\n"),cmd);
+   return false;
 }
 
 void CmdExec::print_cmd_index()
@@ -2611,12 +2664,14 @@ CMD(help)
 {
    if(args->count()>1)
    {
+      exit_code=0;
       for(;;)
       {
 	 const char *cmd=args->getnext();
 	 if(cmd==0)
 	    break;
-	 parent->print_cmd_help(cmd);
+	 if(!parent->print_cmd_help(cmd))
+	    exit_code=1;
       }
       return 0;
    }
@@ -2836,7 +2891,13 @@ CMD(bookmark)
    else if(!strcasecmp(op,"edit"))
    {
       lftp_bookmarks.Remove(""); // force bookmark file creation
-      parent->PrependCmd("shell \"/bin/sh -c 'exec ${EDITOR:-vi} ${LFTP_HOME:-$HOME/.lftp}/bookmarks'\"\n");
+
+      xstring cmd0("exec ${EDITOR:-vi} ");
+      cmd0.append(shell_encode(lftp_bookmarks.GetFilePath()));
+      xstring cmd1("/bin/sh -c ");
+      cmd1.append(shell_encode(cmd0));
+
+      parent->PrependCmd(xstring::get_tmp("shell ").append_quoted(cmd1));
    }
    else if(!strcasecmp(op,"import"))
    {
@@ -2984,6 +3045,7 @@ CMD(du)
 	 break;
       case 'b':
 	 blocksize = 1;
+	 human_opts = 0;
 	 break;
       case 'c':
 	 print_totals=true;
@@ -3001,18 +3063,18 @@ CMD(du)
 	 file_count=true;
 	 break;
       case 'h':
-	 blocksize=1;
-	 human_opts|=human_autoscale|human_SI|human_base_1024;
+	 human_opts |= human_autoscale|human_SI|human_base_1024;
 	 break;
       case 'H':
-	 blocksize=1;
 	 human_opts |= human_autoscale|human_SI;
 	 break;
       case 'k': /* the default; here for completeness */
 	 blocksize = 1024;
+	 human_opts = 0;
 	 break;
       case 'm':
 	 blocksize = 1024*1024;
+	 human_opts = 0;
 	 break;
       case 's':
 	 summarize_only = true;
