@@ -337,7 +337,9 @@ static ResType lftp_vars[] = {
    {"mirror:dereference",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:skip-noaccess",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:no-empty-dirs",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"mirror:require-source",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
 
+   {"sftp:auto-confirm",	 "no",	  ResMgr::BoolValidate,0},
    {"sftp:max-packets-in-flight","16",	  ResMgr::UNumberValidate,0},
    {"sftp:protocol-version",	 "6",	  ResMgr::UNumberValidate,0},
    {"sftp:size-read",		 "32k",	  ResMgr::UNumberValidate,0},
@@ -348,6 +350,7 @@ static ResType lftp_vars[] = {
    {"sftp:use-full-path",	 "yes",	  ResMgr::BoolValidate,0},
 
    {"file:charset",		 "",	  ResMgr::CharsetValidate,ResMgr::NoClosure},
+   {"file:use-lock",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
 
    {"dns:cache-enable",		 "yes",	  ResMgr::BoolValidate,0},
    {"dns:cache-expire",		 "1h",	  ResMgr::TimeIntervalValidate,0},
@@ -366,6 +369,7 @@ static ResType lftp_vars[] = {
    {"dns:strict-dnssec",	 "no",	  ResMgr::BoolValidate,0},
 #endif
 
+   {"fish:auto-confirm",	 "no",	  ResMgr::BoolValidate,0},
    {"fish:shell",		 "/bin/sh",0,0},
    {"fish:connect-program",	 "ssh -a -x",0,0},
    {"fish:charset",		 "",	  ResMgr::CharsetValidate,0},
@@ -379,6 +383,8 @@ static ResType lftp_vars[] = {
    {"xfer:auto-rename",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:log-file",              "",     ResMgr::FileCreatable,ResMgr::NoClosure},
    {"xfer:max-log-size",	 "1M",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
+   {"xfer:use-temp-file",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:temp-file-name",	 ".in.*", 0,ResMgr::NoClosure},
 
 #if USE_SSL
    {"ssl:ca-file",		 "",	  ResMgr::FileReadable,ResMgr::NoClosure},
@@ -388,6 +394,7 @@ static ResType lftp_vars[] = {
    {"ssl:check-hostname",	 "yes",	  ResMgr::BoolValidate,0},
    {"ssl:verify-certificate",	 "yes",	  ResMgr::BoolValidate,0},
    {"ssl:use-sni",		 "yes",	  ResMgr::BoolValidate,0},
+   {"ssl:priority",		 "",	  0,0},
 # if USE_OPENSSL
    {"ssl:ca-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
    {"ssl:crl-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
